# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Pytest configuration."""

from __future__ import absolute_import, print_function

import os
from datetime import datetime

import pytest
from flask import Blueprint, Flask
from flask_celeryext import FlaskCeleryExt
from six import StringIO

from invenio_mail import InvenioMail, config


@pytest.fixture(scope='session')
def email_task_app(request):
    """Flask application fixture."""
    app = Flask('testapp')
    app.config.update(
        SQLALCHEMY_DATABASE_URI=os.environ.get(
            'SQLALCHEMY_DATABASE_URI', 'sqlite://'
        ),
        CELERY_ALWAYS_EAGER=True,
        CELERY_RESULT_BACKEND='cache',
        CELERY_CACHE_BACKEND='memory',
        CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
        MAIL_SUPPRESS_SEND=True
    )
    FlaskCeleryExt(app)

    InvenioMail(app, StringIO())

    return app


@pytest.fixture(scope='session')
def email_api_app(email_task_app):
    """Flask application fixture."""
    email_task_app.register_blueprint(
        Blueprint('invenio_mail', __name__, template_folder='templates')
    )

    return email_task_app


@pytest.fixture
def email_params():
    """Email parameters fixture."""
    return {
        'subject': 'subject',
        'recipients': ['recipient@inveniosoftware.com'],
        'sender': 'sender@inveniosoftware.com',
        'cc': 'cc@inveniosoftware.com',
        'bcc': 'bcc@inveniosoftware.com',
        'reply_to': 'reply_to@inveniosoftware.com',
        'date': datetime.now(),
        'attachments': [],
        'charset': None,
        'extra_headers': None,
        'mail_options': [],
        'rcpt_options': [],
    }


@pytest.fixture
def email_ctx():
    """Email context fixture."""
    return {
        'user': 'User',
        'content': 'This a content.',
        'sender': 'sender',
    }



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Configuration for Invenio-Mail.

Invenio-Mail is a tiny integration layer between Invenio and Flask-Mail, so
please refer to
`Flask-Mail <https://pythonhosted.org/Flask-Mail/#configuring-flask-mail>`_'s
list of configuration variables.
"""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test package API."""

from __future__ import absolute_import, print_function

from invenio_mail.api import TemplatedMessage


def test_templated_message(email_api_app, email_params, email_ctx):
    """Test that all the fields given are inside the message."""
    with email_api_app.app_context():
        msg = TemplatedMessage(template_body='invenio_mail/base.txt',
                               template_html='invenio_mail/base.html',
                               ctx=email_ctx, **email_params)

        for key in email_params:
            assert email_params[key] == getattr(msg, key), key

        # let's check that the body and html are correctly formatted
        assert '<p>Dear {0},</p>'.format(email_ctx['user']) in msg.html
        assert 'Dear {0},'.format(email_ctx['user']) in msg.body
        assert '<p>{0}</p>'.format(email_ctx['content']) in msg.html
        assert '{0}'.format(email_ctx['content']) in msg.body
        assert email_ctx['sender'] in msg.html
        assert email_ctx['sender'] in msg.body


def test_simple_templated_message(email_api_app):
    """Test that defaults are sane."""
    with email_api_app.app_context():
        msg = TemplatedMessage(template_body='invenio_mail/base.txt',
                               template_html='invenio_mail/base.html')
        assert msg



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Background tasks for mail module."""

from __future__ import absolute_import, print_function

from celery import shared_task
from flask import current_app
from flask_mail import Message


@shared_task
def send_email(data):
    """Celery task for sending emails.

    .. warning::

       Due to an incompatibility between MessagePack serialization and Message,
       support for attachments and dates is limited. Consult the tests for
       details.
    """
    msg = Message()
    msg.__dict__.update(data)
    current_app.extensions['mail'].send(msg)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Template based messages."""

from __future__ import absolute_import, print_function

from flask import render_template
from flask_mail import Message


class TemplatedMessage(Message):
    """Siplify creation of templated messages."""

    def __init__(self, template_body=None, template_html=None, ctx=None,
                 **kwargs):
        r"""Build message body and HTML based on provided templates.

        Provided templates can use keyword arguments ``body`` and ``html``
        respectively.

        :param template_body: Path to the text template.
        :param template_html: Path to the html template.
        :param ctx: A mapping containing additional information passed to the
            template.
        :param \*\*kwargs: Keyword arguments as defined in
            :class:`flask_mail.Message`.
        """
        ctx = ctx if ctx else {}
        if template_body:
            kwargs['body'] = render_template(
                template_body, body=kwargs.get('body'), **ctx
            )
        if template_html:
            kwargs['html'] = render_template(
                template_html, html=kwargs.get('html'), **ctx
            )
        super(TemplatedMessage, self).__init__(**kwargs)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio mail module.

The Invenio-Mail module is a tiny wrapper around Flask-Mail that provides
printing of emails to standard output when the configuration variable
``MAIL_SUPPRESS_SEND`` is ``True``.

Invenio-Mail also takes care of initializing Flask-Mail if not already
initialized.

First, initialize the extension:

>>> from flask import Flask
>>> from invenio_mail import InvenioMail
>>> app = Flask('myapp')
>>> app.config.update(MAIL_SUPPRESS_SEND=True)
>>> InvenioMail(app)
<invenio_mail.ext.InvenioMail ...>

Next, let's send an email:

>>> from flask_mail import Message
>>> msg = Message('Hello', sender='from@example.org',
...    recipients=['to@example.com'], body='Hello, World!')
>>> with app.app_context():
...     app.extensions['mail'].send(msg)
Content-Type: text/plain; charset="utf-8"...

Template based messages
-----------------------
A simple API lets you create a message from a template, so you just have to
give the right arguments to get the full message. Moreover, it can create
a complete e-mail with both HTML and text content.

First, you need to instantiate the :class:`~invenio_mail.api.TemplatedMessage`
class, just like you would do with a standard :class:`flask_mail.Message`:

>>> from invenio_mail.api import TemplatedMessage
>>> with app.app_context():
...    msg = TemplatedMessage(
...         template_html='', # path to your template
...         template_body='', # path to your template
...         subject='Hello',
...         sender='from@example.org',
...         recipients=['to@example.com'],
...         ctx={
...             'content': 'Hello, World!',
...             'logo': 'logo.png',
...             'sender': 'Sender',
...             'user': 'User',
...         })

You just need to add the templates to use and a ``ctx`` dictionary,
containing the values useful to fill the templates. If you ommit these three
arguments, you will have the same result as you would with the standard
:class:`flask_mail.Message` class.

Note that you must be in the application context in order to be able to render
the templates. Once you have created a message, you can send it the standard
way:

>>> with app.app_context():
...     app.extensions['mail'].send(msg)
Content-Type: text/plain; charset="utf-8"...


Writing extensions
------------------
By default you should just depend on Flask-Mail if you are writing an
extension which needs email sending functionality:

.. code-block:: python

   from flask import current_app
   from flask_mail import Message

   def mystuff():
       msg = Message('Hello', sender='from@example.org',
                     recipients=['to@example.com'], body='Hello, World!')
       current_app.extensions['mail'].send(msg)


Remember to add Flask-Mail to your ``setup.py`` file as well:

.. code-block:: python

   setup(
       # ...
       install_requires = ['Flask-Mail>=0.9.1',]
       #...
    )
"""

from __future__ import absolute_import, print_function

from .ext import InvenioMail
from .version import __version__

__all__ = ('__version__', 'InvenioMail')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Module tests."""

from __future__ import absolute_import, print_function

import os

import pkg_resources
from flask_mail import Attachment

from invenio_mail.tasks import send_email


def test_send_message_outbox(email_task_app):
    """Test sending a message using Task module."""
    with email_task_app.app_context():
        with email_task_app.extensions['mail'].record_messages() as outbox:
            msg = {
                'subject': 'Test1',
                'sender': 'test1@test1.test1',
                'recipients': ['test1@test1.test1']
            }

            send_email.delay(msg)

            assert len(outbox) == 1
            sent_msg = outbox[0]
            assert sent_msg.subject == 'Test1'
            assert sent_msg.sender == 'test1@test1.test1'
            assert sent_msg.recipients == ['test1@test1.test1']


def test_send_message_stream(email_task_app):
    """Test sending a message using Task module."""
    with email_task_app.app_context():
        with email_task_app.extensions['mail'].record_messages() as outbox:
            msg = {
                'subject': 'Test2',
                'sender': 'test2@test2.test2',
                'recipients': ['test2@test2.test2']
            }

            send_email.delay(msg)

            result_stream = email_task_app.extensions['invenio-mail'].stream
            assert result_stream.getvalue().find(
                'From: test2@test2.test2') != -1
            assert result_stream.getvalue().find('To: test2@test2.test2') != -1
            assert result_stream.getvalue().find('Subject: Test2') != -1


def test_send_message_with_attachments(email_task_app):
    """Test sending a message with attachments."""
    with email_task_app.app_context():
        filename = pkg_resources.resource_filename(
            __name__, os.path.join('attachments', 'invenio.svg'))
        content_type = 'image/svg+xml'
        data = pkg_resources.resource_string(
            __name__, os.path.join('attachments', 'invenio.svg'))

        attachments = [Attachment(filename, content_type, data)]
        msg = {
            'subject': 'Test3',
            'sender': 'test3@test3.test3',
            'recipients': ['test3@test3.test3'],
            'attachments': attachments
        }

        send_email.delay(msg)

        result_stream = email_task_app.extensions['invenio-mail'].stream
        assert result_stream.getvalue().find(
            'Content-Transfer-Encoding: base64') != -1
        assert result_stream.getvalue().find(
            'Content-Disposition: attachment;') != -1


def test_send_message_with_date(email_task_app):
    """Test sending a message with a date."""
    with email_task_app.app_context():
        msg = {
            'subject': 'Test4',
            'sender': 'test4@test4.test4',
            'recipients': ['test4@test4.test4'],
            'date': 1456242014.398119
        }

        send_email.delay(msg)

        result_stream = email_task_app.extensions['invenio-mail'].stream
        assert result_stream.getvalue().find('Date: Tue, 23 Feb 2016') != -1



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio mail module."""

from __future__ import absolute_import, print_function

import sys
import threading

from flask_mail import Mail, email_dispatched


def print_email(message, app):
    """Print mail to stream.

    Signal handler for email_dispatched signal. Prints by default the output
    to the stream specified in the constructor of InvenioMail.

    :param message: Message object.
    :param app: Flask application object.
    """
    invenio_mail = app.extensions['invenio-mail']
    with invenio_mail._lock:
        invenio_mail.stream.write(
            '{0}\n{1}\n'.format(message.as_string(), '-' * 79))
        invenio_mail.stream.flush()


class InvenioMail(object):
    """Invenio-Mail extension."""

    def __init__(self, app=None, stream=None):
        """Extension initialization.

        Mails are only printed to the stream if ``MAIL_SUPPRESS_SEND`` is
        ``True``.

        :param app: Flask application object.
        :param stream: Stream to print emails to. Defaults to ``sys.stdout``.
        """
        self.stream = stream or sys.stdout
        self._lock = threading.RLock()
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization.

        The initialization will:

         * Set default values for the configuration variables.
         * Initialise the Flask mail extension.
         * Configure the extension to avoid the email sending in case of debug
           or ``MAIL_SUPPRESS_SEND`` config variable set. In this case, the
           email will be written in the stream configured in the extension.

        :param app: Flask application object.
        """
        self.init_config(app)
        if 'mail' not in app.extensions:
            Mail(app)
        if app.config.get('MAIL_SUPPRESS_SEND', False) or app.debug:
            email_dispatched.connect(print_email)
        app.extensions['invenio-mail'] = self

    @staticmethod
    def init_config(app):
        """Initialize configuration.

        :param app: Flask application object.
        """
        app.config.setdefault('MAIL_DEBUG', app.debug)
        app.config.setdefault('MAIL_SUPPRESS_SEND', app.debug or app.testing)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Module tests."""

from __future__ import absolute_import, print_function

import sys

from flask import Flask
from flask_mail import Message

from invenio_mail import InvenioMail

PY3 = sys.version_info[0] == 3

if PY3:
    from io import StringIO
else:
    from StringIO import StringIO


def test_version():
    """Test version import."""
    from invenio_mail import __version__
    assert __version__


def test_init():
    """Test extension initialization."""
    app = Flask('testapp')
    ext = InvenioMail(app)
    assert 'invenio-mail' in app.extensions

    app = Flask('testapp')
    ext = InvenioMail()
    assert 'invenio-mail' not in app.extensions
    ext.init_app(app)
    assert 'invenio-mail' in app.extensions


def test_print_email():
    """Test printing of email."""
    app = Flask('testapp')
    app.config.update(MAIL_SUPPRESS_SEND=True)

    output = StringIO()
    InvenioMail(app, stream=output)

    with app.app_context():
        msg = Message(
            'Test subject', sender='from@example.com',
            recipients=['to@example.com'], body='Test Body')
        app.extensions['mail'].send(msg)

    email = output.getvalue()
    output.close()

    sep = '\r\n' if PY3 else '\n'

    assert 'Subject: Test subject{0}'.format(sep) in email
    assert 'From: from@example.com{0}'.format(sep) in email
    assert 'To: to@example.com{0}'.format(sep) in email
    assert '{0}{0}Test Body'.format(sep) in email
